==> af-common-expression-2.0.10-sources.jar: com/af/v4/system/common/expression/report/core/HeadBlock.java <==
package com.af.v4.system.common.expression.report.core;

import org.dom4j.Element;
import org.json.JSONArray;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

// 一批单元格组成的可以复制的表头块，每个块对应一个数据集
// content - 重复快用于获取数据的表达式
public class HeadBlock extends Block {
    // 块头单元格，块可以没有头单元格
    public Cell headCell;

    // 重复数据的名称
    public String name;

    // 块单元格中，表达式里data对应的对象
    public Object data;

    // 从xml元素构造
    public HeadBlock(Element ele) {
        super(ele);
        // 加载子块
        for (Element obj : ele.elements("headblock")) {
            this.cells.add(new HeadBlock(obj));
        }
        // 获取重复数据的名称
        this.name = ele.attributeValue("name");

        // 加载头
        Element eHead = ele.element("head");
        if (eHead != null) {
            this.headCell = new Cell(eHead);
        }
    }

    public HeadBlock(int row, int column, int rowSpan, int columnSpan,
                     String content) {
        super(row, column, rowSpan, columnSpan, content);
    }

    // 重置块的起始列
    public void setColumn(int column) {
        // 对块中的每一个单元格，更改列号，如果是块，要递归做
        for (Cell cell : cells) {
            cell.column = cell.column + column;
            if (cell instanceof HeadBlock block) {
                block.setColumn(column);
            }
        }
        if (this.headCell != null) {
            this.headCell.column += column;
        }
    }

    // 重置块的起始行
    public void setRow(int row) {
        // 对块中的每一个单元格，更改列号，如果是块，要递归做
        for (Cell cell : cells) {
            cell.row = cell.row + row;
            if (cell instanceof HeadBlock block) {
                block.setColumn(row);
            }
        }
        if (this.headCell != null) {
            this.headCell.row += row;
        }
    }

    // 对单元格进行编译
    @Override
    public void compile() {
        super.compile();
        if (this.headCell != null) {
            this.headCell.compile();
        }
    }

    @Override
    public void setReport(Report report) {
        super.setReport(report);
        if (this.headCell != null) {
            this.headCell.setReport(report);
        }
    }

    // 根据块的数据，产生可以使用的单元格
    // result - 创建的单元格存放在result里
    // return - 复制了多少列
    public int createCell(List<Cell> result) {
        // 对单元格按行列进行排序，把同一行放在一起
        cells.sort(Comparator.comparingInt(o -> (o.row * 10000 + o.column)));

        // 复制了多少列
        int copyColumn = 0;
        // 对所有排序了的单元格（包括块），重新计算列
        for (Cell cell : cells) {
            // 如果是block
            if (cell instanceof HeadBlock block) {
                // 如果是可复制的block
                if (block.content != null && !block.content.trim().isEmpty()) {
                    int i = 0;
                    Object v = block.invoke();
                    if (!(v instanceof JSONArray data)) {
                        throw new RuntimeException("必须返回JSONArray，行："
                                + block.row + ", 列：" + block.column + ", 内容："
                                + block.content);
                    }
                    for (Object item : data) {
                        HeadBlock newBlock = block.clone();
                        // 第一块只加增加的列数，第二块要加上块本身的列数
                        if (i != 0) {
                            copyColumn += block.columnSpan;
                        }
                        // 设置块的起始列
                        newBlock.setColumn(copyColumn);
                        // 让新的块产生单元格
                        List<Cell> cells = new ArrayList<>();

                        newBlock.data = item;
                        this.report.vars.put(newBlock.name, item);

                        copyColumn += newBlock.createCell(cells);
                        result.addAll(cells);

                        i++;
                    }
                } else {
                    // 不可复制的block，直接把block内容拿处理即可
                    HeadBlock newBlock = block.clone();
                    // 设置块的起始列
                    newBlock.setColumn(copyColumn);
                    // 让新的块产生单元格
                    List<Cell> cells = new ArrayList<>();
                    copyColumn += newBlock.createCell(cells);
                    result.addAll(cells);
                }
            } else {
                cell.column = cell.column + copyColumn;
                cell.content = cell.invoke().toString();
                result.add(cell);
            }
        }

        // 如果有头部，返回头部单元格扩展的结果
        if (this.headCell != null) {
            this.headCell.columnSpan += copyColumn;
            this.headCell.content = this.headCell.invoke().toString();
            result.add(this.headCell);
        }

        return copyColumn;
    }

    // 复制
    public HeadBlock clone() {
        HeadBlock result = new HeadBlock(this.row, this.column, this.rowSpan,
                this.columnSpan, this.content);
        if (this.delegate != null) {
            result.delegate = this.delegate;
        }
        if (this.report != null) {
            result.report = this.report;
        }

        // 复制单元格
        for (Cell cell : cells) {
            result.cells.add(cell.clone());
        }

        // 复制表头
        if (this.headCell != null) {
            result.headCell = this.headCell.clone();
        }
        if (this.name != null) {
            result.name = this.name;
        }

        return result;
    }
}

==> af-common-expression-2.10.11-sources.jar: com/af/v4/system/common/expression/report/core/HeadBlock.java <==
package com.af.v4.system.common.expression.report.core;

import org.dom4j.Element;
import org.json.JSONArray;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

// 一批单元格组成的可以复制的表头块，每个块对应一个数据集
// content - 重复快用于获取数据的表达式
public class HeadBlock extends Block {
    // 块头单元格，块可以没有头单元格
    public Cell headCell;

    // 重复数据的名称
    public String name;

    // 块单元格中，表达式里data对应的对象
    public Object data;

    // 从xml元素构造
    public HeadBlock(Element ele) {
        super(ele);
        // 加载子块
        for (Element obj : ele.elements("headblock")) {
            this.cells.add(new HeadBlock(obj));
        }
        // 获取重复数据的名称
        this.name = ele.attributeValue("name");

        // 加载头
        Element eHead = ele.element("head");
        if (eHead != null) {
            this.headCell = new Cell(eHead);
        }
    }

    public HeadBlock(int row, int column, int rowSpan, int columnSpan,
                     String content) {
        super(row, column, rowSpan, columnSpan, content);
    }

    // 重置块的起始列
    public void setColumn(int column) {
        // 对块中的每一个单元格，更改列号，如果是块，要递归做
        for (Cell cell : cells) {
            cell.column = cell.column + column;
            if (cell instanceof HeadBlock block) {
                block.setColumn(column);
            }
        }
        if (this.headCell != null) {
            this.headCell.column += column;
        }
    }

    // 重置块的起始行
    public void setRow(int row) {
        // 对块中的每一个单元格，更改列号，如果是块，要递归做
        for (Cell cell : cells) {
            cell.row = cell.row + row;
            if (cell instanceof HeadBlock block) {
                block.setColumn(row);
            }
        }
        if (this.headCell != null) {
            this.headCell.row += row;
        }
    }

    // 对单元格进行编译
    @Override
    public void compile() {
        super.compile();
        if (this.headCell != null) {
            this.headCell.compile();
        }
    }

    @Override
    public void setReport(Report report) {
        super.setReport(report);
        if (this.headCell != null) {
            this.headCell.setReport(report);
        }
    }

    // 根据块的数据，产生可以使用的单元格
    // result - 创建的单元格存放在result里
    // return - 复制了多少列
    public int createCell(List<Cell> result) {
        // 对单元格按行列进行排序，把同一行放在一起
        cells.sort(Comparator.comparingInt(o -> (o.row * 10000 + o.column)));

        // 复制了多少列
        int copyColumn = 0;
        // 对所有排序了的单元格（包括块），重新计算列
        for (Cell cell : cells) {
            // 如果是block
            if (cell instanceof HeadBlock block) {
                // 如果是可复制的block
                if (block.content != null && !block.content.trim().isEmpty()) {
                    int i = 0;
                    Object v = block.invoke();
                    if (v instanceof JSONArray) {
                        for (Object item : (JSONArray) v) {
                            HeadBlock newBlock = block.clone();
                            // 第一块只加增加的列数，第二块要加上块本身的列数
                            if (i != 0) {
                                copyColumn += block.columnSpan;
                            }
                            // 设置块的起始列
                            newBlock.setColumn(copyColumn);
                            // 让新的块产生单元格
                            List<Cell> cells = new ArrayList<>();

                            newBlock.data = item;
                            this.report.vars.put(newBlock.name, item);

                            copyColumn += newBlock.createCell(cells);
                            result.addAll(cells);

                            i++;
                        }
                    } else {
                        throw new RuntimeException("必须返回JSONArray，行："
                                + block.row + ", 列：" + block.column + ", 内容："
                                + block.content);
                    }

                } else {
                    // 不可复制的block，直接把block内容拿处理即可
                    HeadBlock newBlock = block.clone();
                    // 设置块的起始列
                    newBlock.setColumn(copyColumn);
                    // 让新的块产生单元格
                    List<Cell> cells = new ArrayList<>();
                    copyColumn += newBlock.createCell(cells);
                    result.addAll(cells);
                }
            } else {
                cell.column = cell.column + copyColumn;
                cell.content = cell.invoke().toString();
                result.add(cell);
            }
        }

        // 如果有头部，返回头部单元格扩展的结果
        if (this.headCell != null) {
            this.headCell.columnSpan += copyColumn;
            this.headCell.content = this.headCell.invoke().toString();
            result.add(this.headCell);
        }

        return copyColumn;
    }

    // 复制
    public HeadBlock clone() {
        HeadBlock result = new HeadBlock(this.row, this.column, this.rowSpan,
                this.columnSpan, this.content);
        if (this.delegate != null) {
            result.delegate = this.delegate;
        }
        if (this.report != null) {
            result.report = this.report;
        }

        // 复制单元格
        for (Cell cell : cells) {
            result.cells.add(cell.clone());
        }

        // 复制表头
        if (this.headCell != null) {
            result.headCell = this.headCell.clone();
        }
        if (this.name != null) {
            result.name = this.name;
        }

        return result;
    }
}
